@@ -19,6 +19,7 @@
 #include "dupe.h"
 #include "filelist.h"
 #include "img-view.h"
+#include "info.h"
 #include "layout.h"
 #include "layout_image.h"
 #include "menu.h"
@@ -491,6 +492,13 @@ static void collection_table_popup_edit_cb(GtkWidget *widget, gpointer data)
 		}
 }
 
+static void collection_table_popup_info_cb(GtkWidget *widget, gpointer data)
+{
+	CollectTable *ct = data;
+
+	info_window_new(NULL, collection_table_selection_get_list(ct));
+}
+
 static void collection_table_popup_copy_cb(GtkWidget *widget, gpointer data)
 {
 	CollectTable *ct = data;
@@ -617,32 +625,50 @@ static GtkWidget *collection_table_popup_menu(CollectTable *ct, gint over_icon)
 
 	if (over_icon)
 		{
-		menu_item_add(menu, _("View"), collection_table_popup_view_cb, ct);
-		menu_item_add(menu, _("View in new window"), collection_table_popup_view_new_cb, ct);
+		menu_item_add(menu, _("View"),
+			      GTK_SIGNAL_FUNC(collection_table_popup_view_cb), ct);
+		menu_item_add(menu, _("View in new window"),
+			      GTK_SIGNAL_FUNC(collection_table_popup_view_new_cb), ct);
 		menu_item_add_divider(menu);
-		menu_item_add(menu, _("Remove"), collection_table_popup_remove_cb, ct);
+		menu_item_add(menu, _("Remove"),
+			      GTK_SIGNAL_FUNC(collection_table_popup_remove_cb), ct);
 		}
-	menu_item_add(menu, _("Append from file list"), collection_table_popup_add_filelist_cb, ct);
-	menu_item_add(menu, _("Append from collection..."), collection_table_popup_add_collection_cb, ct);
+	menu_item_add(menu, _("Append from file list"),
+		      GTK_SIGNAL_FUNC(collection_table_popup_add_filelist_cb), ct);
+	menu_item_add(menu, _("Append from collection..."),
+		      GTK_SIGNAL_FUNC(collection_table_popup_add_collection_cb), ct);
 	menu_item_add_divider(menu);
-	menu_item_add(menu, _("Select all"), collection_table_popup_selectall_cb, ct);
-	menu_item_add(menu, _("Select none"), collection_table_popup_unselectall_cb, ct);
+	menu_item_add(menu, _("Select all"),
+		      GTK_SIGNAL_FUNC(collection_table_popup_selectall_cb), ct);
+	menu_item_add(menu, _("Select none"),
+		      GTK_SIGNAL_FUNC(collection_table_popup_unselectall_cb), ct);
 	menu_item_add_divider(menu);
 	if (over_icon)
 		{
-		submenu_add_edit(menu, NULL, collection_table_popup_edit_cb, ct);
-		menu_item_add(menu, _("Copy..."), collection_table_popup_copy_cb, ct);
-		menu_item_add(menu, _("Move..."), collection_table_popup_move_cb, ct);
-		menu_item_add(menu, _("Rename..."), collection_table_popup_rename_cb, ct);
-		menu_item_add(menu, _("Delete..."), collection_table_popup_delete_cb, ct);
+		submenu_add_edit(menu, NULL,
+				 GTK_SIGNAL_FUNC(collection_table_popup_edit_cb), ct);
+		menu_item_add(menu, _("Properties"),
+			      GTK_SIGNAL_FUNC(collection_table_popup_info_cb), ct);
+		menu_item_add_divider(menu);
+		menu_item_add(menu, _("Copy..."),
+			      GTK_SIGNAL_FUNC(collection_table_popup_copy_cb), ct);
+		menu_item_add(menu, _("Move..."),
+			      GTK_SIGNAL_FUNC(collection_table_popup_move_cb), ct);
+		menu_item_add(menu, _("Rename..."),
+			      GTK_SIGNAL_FUNC(collection_table_popup_rename_cb), ct);
+		menu_item_add(menu, _("Delete..."),
+			      GTK_SIGNAL_FUNC(collection_table_popup_delete_cb), ct);
 		menu_item_add_divider(menu);
 		}
-	submenu_add_sort(menu, collection_table_popup_sort_cb, ct, FALSE, TRUE, FALSE, 0);
+	submenu_add_sort(menu, GTK_SIGNAL_FUNC(collection_table_popup_sort_cb), ct, FALSE, TRUE, FALSE, 0);
 	menu_item_add_divider(menu);
-	menu_item_add(menu, _("Save collection"), collection_table_popup_save_cb, ct);
-	menu_item_add(menu, _("Save collection as..."), collection_table_popup_save_as_cb, ct);
+	menu_item_add(menu, _("Save collection"),
+		      GTK_SIGNAL_FUNC(collection_table_popup_save_cb), ct);
+	menu_item_add(menu, _("Save collection as..."),
+		      GTK_SIGNAL_FUNC(collection_table_popup_save_as_cb), ct);
 	menu_item_add_divider(menu);
-	menu_item_add(menu, _("Find duplicates..."), collection_table_popup_find_dupes_cb, ct);
+	menu_item_add(menu, _("Find duplicates..."),
+		      GTK_SIGNAL_FUNC(collection_table_popup_find_dupes_cb), ct);
 
 	return menu;
 }
@@ -1131,13 +1157,13 @@ static void collection_table_drag_scroll(CollectTable *ct, gint scroll)
  *-------------------------------------------------------------------
  */
 
-static void clist_button_motion(GtkWidget *widget, GdkEventButton *bevent, gpointer data)
+static gint clist_button_motion(GtkWidget *widget, GdkEventButton *bevent, gpointer data)
 {
 	CollectTable *ct = data;
 	gint row, col;
 	CollectInfo *info;
 
-	if (bevent->window != GTK_CLIST(ct->clist)->clist_window) return;
+	if (bevent->window != GTK_CLIST(ct->clist)->clist_window) return FALSE;
 
 	gtk_clist_get_selection_info(GTK_CLIST(ct->clist), bevent->x, bevent->y, &row, &col);
 
@@ -1147,10 +1173,10 @@ static void clist_button_motion(GtkWidget *widget, GdkEventButton *bevent, gpoin
 		{
 		tip_update(ct, info);
 		collection_table_drag_scroll(ct, FALSE);
-		return;
+		return FALSE;
 		}
 
-	if (!info || info == ct->mouse_current) return;
+	if (!info || info == ct->mouse_current) return FALSE;
 
 	gtk_clist_freeze(GTK_CLIST(ct->clist));
 
@@ -1163,9 +1189,11 @@ static void clist_button_motion(GtkWidget *widget, GdkEventButton *bevent, gpoin
 	collection_table_select_region_util(ct, ct->mouse_start, ct->mouse_current, ct->mouse_add);
 
 	gtk_clist_thaw(GTK_CLIST(ct->clist));
+
+	return FALSE;
 }
 
-static void clist_button_press(GtkWidget *widget, GdkEventButton *bevent, gpointer data)
+static gint clist_button_press(GtkWidget *widget, GdkEventButton *bevent, gpointer data)
 {
 	CollectTable *ct = data;
 	gint row, col;
@@ -1174,7 +1202,7 @@ static void clist_button_press(GtkWidget *widget, GdkEventButton *bevent, gpoint
 
 	tip_unschedule(ct);
 
-	if (bevent->window != GTK_CLIST(ct->clist)->clist_window) return;
+	if (bevent->window != GTK_CLIST(ct->clist)->clist_window) return FALSE;
 
 	gtk_clist_get_selection_info(GTK_CLIST(ct->clist), bevent->x, bevent->y, &row, &col);
 	info = collection_table_find_info(ct, row, col);
@@ -1188,11 +1216,11 @@ static void clist_button_press(GtkWidget *widget, GdkEventButton *bevent, gpoint
 				GTK_CLIST(ct->clist)->focus_row = ct->focus_row;
 				}
 
-			if (!info) return;
+			if (!info) return FALSE;
 			if (bevent->type == GDK_2BUTTON_PRESS)
 				{
 				layout_image_set_collection(NULL, ct->cd, info);
-				return;
+				return FALSE;
 				}
 
 			ct->mouse_in_drag = TRUE;
@@ -1267,23 +1295,27 @@ static void clist_button_press(GtkWidget *widget, GdkEventButton *bevent, gpoint
 		default:
 			break;
 		}
+
+	return FALSE;
 }
 
-static void clist_button_release(GtkWidget *widget, GdkEventButton *bevent, gpointer data)
+static gint clist_button_release(GtkWidget *widget, GdkEventButton *bevent, gpointer data)
 {
 	CollectTable *ct = data;
 
 	tip_schedule(ct);
 
-	if (!ct->mouse_in_drag) return;
+	if (!ct->mouse_in_drag) return FALSE;
 
 	gtk_grab_remove(ct->clist);
 	gdk_pointer_ungrab(bevent->time);
 
 	ct->mouse_in_drag = FALSE;
+
+	return FALSE;
 }
 
-static void clist_button_leave(GtkWidget *widget, GdkEventCrossing *event, gpointer data)
+static gint clist_button_leave(GtkWidget *widget, GdkEventCrossing *event, gpointer data)
 {
 	CollectTable *ct = data;
 
@@ -1291,6 +1323,8 @@ static void clist_button_leave(GtkWidget *widget, GdkEventCrossing *event, gpoin
 
 	collection_table_drag_scroll(ct, FALSE);
 	clist_insert_marker(ct, NULL, FALSE);
+
+	return FALSE;
 }
 
 /*
@@ -1814,15 +1848,16 @@ static GtkWidget *confirm_dir_list(CollectTable *ct, GList *list)
 	d->list = list;
 
 	menu = popup_menu_short_lived();
-	gtk_signal_connect(GTK_OBJECT(menu), "destroy", confirm_dir_list_destroy, d);
+	gtk_signal_connect(GTK_OBJECT(menu), "destroy",
+			   GTK_SIGNAL_FUNC(confirm_dir_list_destroy), d);
 
 	menu_item_add(menu, _("Dropped list includes directories."), NULL, NULL);
 	menu_item_add_divider(menu);
-	menu_item_add(menu, _("Add contents"), confirm_dir_list_add, d);
-	menu_item_add(menu, _("Add contents recursive"), confirm_dir_list_recurse, d);
-	menu_item_add(menu, _("Skip directories"), confirm_dir_list_skip, d);
+	menu_item_add(menu, _("Add contents"), GTK_SIGNAL_FUNC(confirm_dir_list_add), d);
+	menu_item_add(menu, _("Add contents recursive"), GTK_SIGNAL_FUNC(confirm_dir_list_recurse), d);
+	menu_item_add(menu, _("Skip directories"), GTK_SIGNAL_FUNC(confirm_dir_list_skip), d);
 	menu_item_add_divider(menu);
-	menu_item_add(menu, _("Cancel"), confirm_dir_list_cancel, d);
+	menu_item_add(menu, _("Cancel"), GTK_SIGNAL_FUNC(confirm_dir_list_cancel), d);
 
 	return menu;
 }
@@ -1990,7 +2025,7 @@ static void collection_table_dnd_end(GtkWidget *widget, GdkDragContext *context,
 	clist_insert_marker(ct, NULL, FALSE);
 }
 
-static void collection_table_dnd_motion(GtkWidget *widget, GdkDragContext *context,
+static gint collection_table_dnd_motion(GtkWidget *widget, GdkDragContext *context,
 					gint x, gint y, guint time, gpointer data)
 {
 	CollectTable *ct = data;
@@ -2005,30 +2040,42 @@ static void collection_table_dnd_motion(GtkWidget *widget, GdkDragContext *conte
 		{
 		clist_insert_marker(ct, NULL, TRUE);
 		}
+
+	return FALSE;
 }
 
+static void collection_table_dnd_leave(GtkWidget *widget, GdkDragContext *context, guint time, gpointer data)
+{
+	CollectTable *ct = data;
+
+	collection_table_drag_scroll(ct, FALSE);
+	clist_insert_marker(ct, NULL, FALSE);
+}
+ 
 static void collection_table_dnd_init(CollectTable *ct)
 {
 	gtk_drag_source_set(ct->clist, GDK_BUTTON2_MASK,
 			    collection_drag_types, n_collection_drag_types,
 			    GDK_ACTION_COPY | GDK_ACTION_MOVE | GDK_ACTION_LINK);
 #if 0
-	gtk_signal_connect_(GTK_OBJECT(ct->clist), "drag_begin",
-			   collection_table_dnd_begin, ct);
+	gtk_signal_connect(GTK_OBJECT(ct->clist), "drag_begin",
+			   GTK_SIGNAL_FUNC(collection_table_dnd_begin), ct);
 #endif
 	gtk_signal_connect(GTK_OBJECT(ct->clist), "drag_data_get",
-			   collection_table_dnd_data_set, ct);
+			   GTK_SIGNAL_FUNC(collection_table_dnd_data_set), ct);
 	gtk_signal_connect(GTK_OBJECT(ct->clist), "drag_end",
-			   collection_table_dnd_end, ct);
+			   GTK_SIGNAL_FUNC(collection_table_dnd_end), ct);
 
 	gtk_drag_dest_set(ct->clist,
 			  GTK_DEST_DEFAULT_MOTION | GTK_DEST_DEFAULT_HIGHLIGHT | GTK_DEST_DEFAULT_DROP,
 			  collection_drop_types, n_collection_drop_types,
 			  GDK_ACTION_COPY | GDK_ACTION_MOVE | GDK_ACTION_ASK);
 	gtk_signal_connect(GTK_OBJECT(ct->clist), "drag_motion",
-			   collection_table_dnd_motion, ct);
+			   GTK_SIGNAL_FUNC(collection_table_dnd_motion), ct);
+	gtk_signal_connect(GTK_OBJECT(ct->clist), "drag_leave",
+			   GTK_SIGNAL_FUNC(collection_table_dnd_leave), ct);
 	gtk_signal_connect(GTK_OBJECT(ct->clist), "drag_data_received",
-			   collection_table_dnd_data_get, ct);
+			   GTK_SIGNAL_FUNC(collection_table_dnd_data_get), ct);
 }
 
 /*
@@ -2098,24 +2145,27 @@ CollectTable *collection_table_new(CollectionData *cd)
 
 	/* we roll our own selection, connect to select row, then unselect it */
 	gtk_clist_set_selection_mode(GTK_CLIST(ct->clist), GTK_SELECTION_SINGLE);
-	gtk_signal_connect(GTK_OBJECT(ct->clist), "select_row", collection_table_select_row_cb, ct);
+	gtk_signal_connect(GTK_OBJECT(ct->clist), "select_row",
+			   GTK_SIGNAL_FUNC(collection_table_select_row_cb), ct);
 
 	gtk_container_add(GTK_CONTAINER(ct->scrolled), ct->clist);
 	gtk_widget_show(ct->clist);
 	
-	gtk_signal_connect(GTK_OBJECT(ct->clist), "destroy", collection_table_destroy, ct);
-	gtk_signal_connect(GTK_OBJECT(ct->clist), "size_allocate", collection_table_sized, ct);
+	gtk_signal_connect(GTK_OBJECT(ct->clist), "destroy",
+			   GTK_SIGNAL_FUNC(collection_table_destroy), ct);
+	gtk_signal_connect(GTK_OBJECT(ct->clist), "size_allocate",
+			   GTK_SIGNAL_FUNC(collection_table_sized), ct);
 
 	collection_table_dnd_init(ct);
 
 	gtk_signal_connect(GTK_OBJECT(ct->clist),"motion_notify_event",
-			   clist_button_motion, ct);
+			   GTK_SIGNAL_FUNC(clist_button_motion), ct);
 	gtk_signal_connect(GTK_OBJECT(ct->clist),"button_press_event",
-                           clist_button_press, ct);
+                           GTK_SIGNAL_FUNC(clist_button_press), ct);
 	gtk_signal_connect(GTK_OBJECT(ct->clist),"button_release_event",
-			   clist_button_release, ct);
+			   GTK_SIGNAL_FUNC(clist_button_release), ct);
 	gtk_signal_connect(GTK_OBJECT(ct->clist),"leave_notify_event",
-			   clist_button_leave, ct);
+			   GTK_SIGNAL_FUNC(clist_button_leave), ct);
 	gtk_widget_set_events(ct->clist, GDK_POINTER_MOTION_MASK | GDK_BUTTON_RELEASE_MASK |
 			      GDK_BUTTON_PRESS_MASK | GDK_LEAVE_NOTIFY_MASK);
 
