@@ -14,10 +14,29 @@
 #define FILELIST_H
 
 
-gint ishidden(const gchar *name);
+typedef struct _FilterEntry FilterEntry;
+struct _FilterEntry {
+	gchar *description;
+	gchar *extensions;
+	gint enabled;
+};
+
+/* you can change, but not add or remove entries from the returned list */
+GList *filter_get_list(void);
+void filter_remove_entry(FilterEntry *fe);
+
+void filter_add(const gchar *description, const gchar *extensions, gint enabled);
+void filter_add_defaults(void);
+void filter_reset(void);
+void filter_rebuild(void);
 
 gint filter_name_exists(const gchar *name);
-void filter_rebuild(void);
+
+void filter_write_list(FILE *f);
+void filter_parse(const gchar *text);
+
+gint ishidden(const gchar *name);
+
 
 GList *path_list_filter(GList *list, gint is_dir_list);
 
