@@ -589,7 +589,7 @@ void info_window_new(const gchar *path, GList *list)
 	gtk_widget_show(hbox);
 
 	id->button_back = gtk_button_new();
-	arrow = gtk_arrow_new(GTK_ARROW_LEFT, GTK_SHADOW_NONE);
+	arrow = gtk_arrow_new(GTK_ARROW_LEFT, GTK_SHADOW_ETCHED_IN);
 	gtk_container_add(GTK_CONTAINER(id->button_back), arrow);
 	gtk_widget_show(arrow);
 
@@ -600,7 +600,7 @@ void info_window_new(const gchar *path, GList *list)
 	gtk_widget_show(id->button_back);
 
 	id->button_next = gtk_button_new();
-	arrow = gtk_arrow_new(GTK_ARROW_RIGHT, GTK_SHADOW_NONE);
+	arrow = gtk_arrow_new(GTK_ARROW_RIGHT, GTK_SHADOW_ETCHED_IN);
 	gtk_container_add(GTK_CONTAINER(id->button_next), arrow);
 	gtk_widget_show(arrow);
 
