@@ -158,6 +158,54 @@ gboolean pixbuf_to_file_as_png (GdkPixbuf *pixbuf, char *filename)
  *-----------------------------------------------------------------------------
  */
 
+static void pixbuf_copy_block_rotate(guchar *src, gint src_row_stride, gint x, gint y,
+				     guchar *dest, gint dest_row_stride, gint w, gint h,
+				     gint bytes_per_pixel, gint counter_clockwise)
+{
+	gint i, j;
+	guchar *sp;
+	guchar *dp;
+
+	for (i = 0; i < h; i++)
+		{
+		sp = src + ((i + y) * src_row_stride) + (x * bytes_per_pixel);
+		for (j = 0; j < w; j++)
+			{
+			if (counter_clockwise)
+				{
+				dp = dest + ((w - j - 1) * dest_row_stride) + (i * bytes_per_pixel);
+				}
+			else
+				{
+				dp = dest + (j * dest_row_stride) + ((h - i - 1) * bytes_per_pixel);
+				}
+			*(dp++) = *(sp++);	/* r */
+			*(dp++) = *(sp++);	/* g */
+			*(dp++) = *(sp++);	/* b */
+			if (bytes_per_pixel == 4) *(dp) = *(sp++);	/* a */
+			}
+		}
+	
+}
+
+static void pixbuf_copy_block(guchar *src, gint src_row_stride, gint w, gint h,
+			      guchar *dest, gint dest_row_stride, gint x, gint y, gint bytes_per_pixel)
+{
+	gint i;
+	guchar *sp;
+	guchar *dp;
+
+	for (i = 0; i < h; i++)
+		{
+		sp = src + (i * src_row_stride);
+		dp = dest + ((y + i) * dest_row_stride) + (x * bytes_per_pixel);
+		memcpy(dp, sp, w * bytes_per_pixel);
+		}
+}
+
+#define ROTATE_BUFFER_WIDTH 48
+#define ROTATE_BUFFER_HEIGHT 48
+
 /*
  * Returns a copy of pixbuf src rotated 90 degrees clockwise or 90 counterclockwise
  *
@@ -170,10 +218,16 @@ GdkPixbuf *pixbuf_copy_rotate_90(GdkPixbuf *src, gint counter_clockwise)
 	gint dw, dh, drs;
 	guchar *s_pix;
         guchar *d_pix;
+#if 0
 	guchar *sp;
         guchar *dp;
+#endif
 	gint i, j;
 	gint a;
+	GdkPixbuf *buffer;
+        guchar *b_pix;
+	gint brs;
+	gint w, h;
 
 	if (!src) return NULL;
 
@@ -191,6 +245,43 @@ GdkPixbuf *pixbuf_copy_rotate_90(GdkPixbuf *src, gint counter_clockwise)
 
 	a = (has_alpha ? 4 : 3);
 
+	buffer = gdk_pixbuf_new(GDK_COLORSPACE_RGB, has_alpha, 8,
+				ROTATE_BUFFER_WIDTH, ROTATE_BUFFER_HEIGHT);
+	b_pix = gdk_pixbuf_get_pixels(buffer);
+	brs = gdk_pixbuf_get_rowstride(buffer);
+
+	for (i = 0; i < sh; i+= ROTATE_BUFFER_WIDTH)
+		{
+		w = MIN(ROTATE_BUFFER_WIDTH, (sh - i));
+		for (j = 0; j < sw; j += ROTATE_BUFFER_HEIGHT)
+			{
+			gint x, y;
+
+			h = MIN(ROTATE_BUFFER_HEIGHT, (sw - j));
+			pixbuf_copy_block_rotate(s_pix, srs, j, i,
+						 b_pix, brs, h, w,
+						 a, counter_clockwise);
+
+			if (counter_clockwise)
+				{
+				x = i;
+				y = sw - h - j;
+				}
+			else
+				{
+				x = sh - w - i;
+				y = j;
+				}
+			pixbuf_copy_block(b_pix, brs, w, h,
+					  d_pix, drs, x, y, a);
+			}
+		}
+
+	gdk_pixbuf_unref(buffer);
+
+#if 0
+	/* this is the simple version of rotation (roughly 2-4x slower) */
+
 	for (i = 0; i < sh; i++)
 		{
 		sp = s_pix + (i * srs);
@@ -211,6 +302,7 @@ GdkPixbuf *pixbuf_copy_rotate_90(GdkPixbuf *src, gint counter_clockwise)
 			if (has_alpha) *(dp) = *(sp++);	/* a */
 			}
 		}
+#endif
 
 	return dest;
 }
