@@ -1,6 +1,6 @@
 /*
  * GQview
- * (C) 2002 John Ellis
+ * (C) 2003 John Ellis
  *
  * Author: John Ellis
  *
@@ -18,6 +18,7 @@
 #include "img-view.h"
 #include "layout_config.h"
 #include "layout_util.h"
+#include "ui_clist_edit.h"
 #include "ui_tabcomp.h"
 #include "ui_utildlg.h"
 
@@ -67,15 +68,6 @@ static gint enable_thumb_dirs_c;
 static gint use_xvpics_thumbnails_c;
 static gint show_dot_files_c;
 static gint file_filter_disable_c;
-static gint filter_include_jpg_c;
-static gint filter_include_xpm_c;
-static gint filter_include_tif_c;
-static gint filter_include_gif_c;
-static gint filter_include_png_c;
-static gint filter_include_ppm_c;
-static gint filter_include_pgm_c;
-static gint filter_include_pcx_c;
-static gint filter_include_bmp_c;
 static gint slideshow_delay_c;
 static gint slideshow_random_c;
 static gint slideshow_repeat_c;
@@ -97,7 +89,9 @@ static gint zoom_increment_c;
 
 static gint enable_read_ahead_c;
 
+#if 0
 static gint place_dialogs_under_mouse_c;
+#endif
 
 static gint black_window_background_c;
 
@@ -105,7 +99,7 @@ static gint dupe_custom_threshold_c;
 
 static GtkWidget *configwindow = NULL;
 static GtkWidget *startup_path_entry;
-static GtkWidget *custom_filter_entry;
+static GtkWidget *filter_clist = NULL;
 static GtkWidget *editor_name_entry[8];
 static GtkWidget *editor_command_entry[8];
 
@@ -168,6 +162,11 @@ static void zoom_increment_cb(GtkObject *adj, gpointer data)
 	zoom_increment_c = (gint)(GTK_ADJUSTMENT(adj)->value * 10.0 + 0.01);
 }
 
+static void cache_clear_cancel_cb(GenericDialog *gd, gpointer data)
+{
+	/* no op, only so cancel button appears */
+}
+
 static void cache_clear_ok_cb(GenericDialog *gd, gpointer data)
 {
 	cache_maintain_home(TRUE);
@@ -178,7 +177,7 @@ static void cache_clear_cb(GtkWidget *widget, gpointer data)
 	GenericDialog *gd;
 
 	gd = generic_dialog_new(_("Clear cache"), _("This will remove all thumbnails that have\nbeen saved to disk, continue?"),
-				"GQview", "clear_cache", TRUE, NULL, NULL);
+				"GQview", "clear_cache", TRUE, cache_clear_cancel_cb, NULL);
 	generic_dialog_add(gd, _("Ok"), cache_clear_ok_cb, TRUE);
 
 	gtk_widget_show(gd->dialog);
@@ -217,24 +216,8 @@ static void config_window_apply(void)
 	buf = gtk_entry_get_text(GTK_ENTRY(startup_path_entry));
 	if (buf && strlen(buf) > 0) startup_path = remove_trailing_slash(buf);
 
-	buf = gtk_entry_get_text(GTK_ENTRY(custom_filter_entry));
-	if ((buf && strlen(buf) > 0) != (custom_filter != NULL)) refresh = TRUE;
-	if ((buf && strlen(buf) > 0 && custom_filter) && strcmp(buf, custom_filter) != 0) refresh = TRUE;
-	g_free(custom_filter);
-	custom_filter = NULL;
-	if (buf && strlen(buf) > 0) custom_filter = g_strdup(buf);
-
 	if (show_dot_files != show_dot_files_c) refresh = TRUE;
 	if (file_filter_disable != file_filter_disable_c) refresh = TRUE;
-	if (filter_include_jpg != filter_include_jpg_c) refresh = TRUE;
-	if (filter_include_xpm != filter_include_xpm_c) refresh = TRUE;
-	if (filter_include_tif != filter_include_tif_c) refresh = TRUE;
-	if (filter_include_gif != filter_include_gif_c) refresh = TRUE;
-	if (filter_include_png != filter_include_png_c) refresh = TRUE;
-	if (filter_include_ppm != filter_include_ppm_c) refresh = TRUE;
-	if (filter_include_pgm != filter_include_pgm_c) refresh = TRUE;
-	if (filter_include_pcx != filter_include_pcx_c) refresh = TRUE;
-	if (filter_include_bmp != filter_include_bmp_c) refresh = TRUE;
 
 	startup_path_enable = startup_path_enable_c;
 	confirm_delete = confirm_delete_c;
@@ -255,15 +238,6 @@ static void config_window_apply(void)
 	use_xvpics_thumbnails = use_xvpics_thumbnails_c;
 	show_dot_files = show_dot_files_c;
 	file_filter_disable = file_filter_disable_c;
-	filter_include_jpg = filter_include_jpg_c;
-	filter_include_xpm = filter_include_xpm_c;
-	filter_include_tif = filter_include_tif_c;
-	filter_include_gif = filter_include_gif_c;
-	filter_include_png = filter_include_png_c;
-	filter_include_ppm = filter_include_ppm_c;
-	filter_include_pgm = filter_include_pgm_c;
-	filter_include_pcx = filter_include_pcx_c;
-	filter_include_bmp = filter_include_bmp_c;
 
 	slideshow_random = slideshow_random_c;
 	slideshow_repeat = slideshow_repeat_c;
@@ -286,7 +260,9 @@ static void config_window_apply(void)
 
 	enable_read_ahead = enable_read_ahead_c;
 
+#if 0
 	place_dialogs_under_mouse = place_dialogs_under_mouse_c;
+#endif
 
 	if (black_window_background != black_window_background_c)
 		{
@@ -338,6 +314,7 @@ static void config_window_close_cb(GtkWidget *widget, gpointer data)
 {
 	gtk_widget_destroy(configwindow);
 	configwindow = NULL;
+	filter_clist = NULL;
 }
 
 static gint config_window_delete(GtkWidget *w, GdkEventAny *event, gpointer data)
@@ -540,6 +517,147 @@ static void add_thumb_size_menu(gchar *text, GtkWidget *box)
 	gtk_widget_show(omenu);
 }
 
+static void filter_clist_populate(void)
+{
+	GList *work;
+	GtkCList *clist;
+
+	if (!filter_clist) return;
+	clist = GTK_CLIST(filter_clist);
+
+	gtk_clist_clear(clist);
+
+	work = filter_get_list();
+	while (work)
+		{
+		FilterEntry *fe;
+		gint row;
+		gchar *buf[4];
+
+		fe = work->data;
+		work = work->next;
+
+		buf[0] = (fe->enabled) ? "X" : "";
+		buf[1] = fe->extensions;
+		buf[2] = fe->description;
+		buf[3] = NULL;
+
+		row = gtk_clist_append(clist, buf);
+		gtk_clist_set_row_data(clist, row, fe);
+		}
+}
+
+static gint filter_clist_ext_cb(ClistEditData *ced,
+				const gchar *old_text, const gchar *new_text, gpointer data)
+{
+	FilterEntry *fe = data;
+
+	if (!fe || !new_text || strlen(new_text) < 1) return FALSE;
+
+	g_free(fe->extensions);
+	fe->extensions = g_strdup(new_text);
+
+	filter_rebuild();
+
+	return TRUE;
+}
+
+static gint filter_clist_desc_cb(ClistEditData *ced,
+				 const gchar *old_text, const gchar *new_text, gpointer data)
+{
+	FilterEntry *fe = data;
+
+	if (!fe || !new_text || strlen(new_text) < 1) return FALSE;
+
+	g_free(fe->description);
+	fe->description = g_strdup(new_text);
+
+	return TRUE;
+}
+
+static gint filter_clist_click_cb(GtkWidget *clist, GdkEventButton *event, gpointer data)
+{
+	gint row = -1;
+	gint column = -1;
+	FilterEntry *fe;
+
+	gtk_clist_get_selection_info(GTK_CLIST(clist), event->x, event->y, &row, &column);
+	if (row == -1 || column == -1) return TRUE;
+
+	fe = gtk_clist_get_row_data(GTK_CLIST(clist), row);
+	gtk_clist_select_row(GTK_CLIST(clist), row, 0);
+	if (!fe || event->button != 1) return TRUE;
+
+	if (column == 0)
+		{
+		fe->enabled = !fe->enabled;
+		gtk_clist_set_text(GTK_CLIST(clist), row, column, (fe->enabled) ? "X" : "");
+		filter_rebuild();
+		}
+	else if (column == 1)
+		{
+		clist_edit_by_row(GTK_CLIST(clist), row, column,
+				  filter_clist_ext_cb, fe);
+		}
+	else if (column == 2)
+		{
+		clist_edit_by_row(GTK_CLIST(clist), row, column,
+				filter_clist_desc_cb, fe);
+		}
+	
+	return TRUE;
+}
+
+static void filter_add_cb(GtkWidget *widget, gpointer data)
+{
+	gint row;
+
+	filter_add("description", ".new", TRUE);
+	filter_clist_populate();
+
+	row = GTK_CLIST(filter_clist)->rows - 1;
+	gtk_clist_moveto(GTK_CLIST(filter_clist), row, 0, 0.0, 1.0);
+	gtk_clist_select_row(GTK_CLIST(filter_clist), row, 0);
+}
+
+static void filter_remove_cb(GtkWidget *widget, gpointer data)
+{
+	gint row;
+	FilterEntry *fe;
+
+	if (!filter_clist || !GTK_CLIST(filter_clist)->selection) return;
+	row = GPOINTER_TO_INT(GTK_CLIST(filter_clist)->selection->data);
+	fe = gtk_clist_get_row_data(GTK_CLIST(filter_clist), row);
+	if (!fe) return;
+
+	filter_remove_entry(fe);
+	filter_rebuild();
+	filter_clist_populate();
+}
+
+static void filer_default_cancel_cb(GenericDialog *gd, gpointer data)
+{
+	/* no op */
+}
+
+static void filer_default_ok_cb(GenericDialog *gd, gpointer data)
+{
+	filter_reset();
+	filter_add_defaults();
+	filter_rebuild();
+	filter_clist_populate();
+}
+
+static void filter_default_cb(GtkWidget *widget, gpointer data)
+{
+	GenericDialog *gd;
+
+	gd = generic_dialog_new(_("Reset filters"), _("This will reset the file filters to the defaults.\nContinue?"),
+				"GQview", "reset_filter", TRUE, filer_default_cancel_cb, NULL);
+	generic_dialog_add(gd, _("Ok"), filer_default_ok_cb, TRUE);
+	gtk_widget_show(gd->dialog);
+}
+
 static void config_window_create(void)
 {
 	GtkWidget *win_vbox;
@@ -549,13 +667,15 @@ static void config_window_create(void)
 	GtkWidget *label;
 	GtkWidget *vbox;
 	GtkWidget *vbox1;
-	GtkWidget *vbox2;
 	GtkWidget *button;
 	GtkWidget *tabcomp;
 	GtkWidget *radiobuttongroup;
 	GtkWidget *table;
 	GtkObject *adj;
 	GtkWidget *spin;
+	GtkWidget *scrolled;
+
+	gchar *filter_titles[] = { "", _("filter"), _("description"), NULL};
 	gint i;
 
 	configwindow = gtk_window_new (GTK_WINDOW_TOPLEVEL);
@@ -888,8 +1008,10 @@ static void config_window_create(void)
         gtk_spin_button_set_update_policy(GTK_SPIN_BUTTON(spin), GTK_UPDATE_ALWAYS);
         gtk_widget_show(spin);
 
+#if 0
 	add_check_button(place_dialogs_under_mouse, &place_dialogs_under_mouse_c,
 			 _("Place dialogs under mouse"), vbox);
+#endif
 
 	add_check_button(tree_descend_subdirs, &tree_descend_subdirs_c,
 			 _("Descend folders in tree view"), vbox);
@@ -929,66 +1051,46 @@ static void config_window_create(void)
 	gtk_widget_show(frame);
 
 	vbox1 = gtk_vbox_new (FALSE, 0);
-	gtk_container_add (GTK_CONTAINER(frame),vbox1);
+	gtk_container_add(GTK_CONTAINER(frame),vbox1);
 	gtk_widget_show(vbox1);
 
-	hbox = gtk_hbox_new (TRUE, 0);
-	gtk_box_pack_start (GTK_BOX(vbox1), hbox, FALSE, FALSE, 0);
+	scrolled = gtk_scrolled_window_new(NULL, NULL);
+	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW (scrolled), GTK_POLICY_AUTOMATIC, GTK_POLICY_ALWAYS);
+	gtk_box_pack_start(GTK_BOX(vbox1), scrolled, TRUE, TRUE, 0);
+	gtk_widget_show(scrolled);
+
+	filter_clist = gtk_clist_new_with_titles(3, filter_titles);
+	gtk_clist_set_selection_mode(GTK_CLIST(filter_clist), GTK_SELECTION_SINGLE);
+	gtk_clist_column_titles_passive(GTK_CLIST(filter_clist));
+	gtk_clist_set_column_auto_resize(GTK_CLIST(filter_clist), 0, TRUE);
+	gtk_clist_set_column_width(GTK_CLIST(filter_clist), 1, 150);
+	filter_clist_populate();
+	gtk_signal_connect(GTK_OBJECT(filter_clist), "button_press_event",
+			   GTK_SIGNAL_FUNC(filter_clist_click_cb), NULL);
+	gtk_container_add(GTK_CONTAINER(scrolled), filter_clist);
+	gtk_widget_show(filter_clist);
+
+	hbox = gtk_hbox_new (FALSE, 5);
+	gtk_box_pack_start (GTK_BOX(vbox1), hbox, FALSE, FALSE, 5);
 	gtk_widget_show(hbox);
 
-	vbox2 = gtk_vbox_new (FALSE, 0);
-	gtk_box_pack_start (GTK_BOX(hbox), vbox2,FALSE, FALSE, 0);
-	gtk_widget_show(vbox2);
+	button = gtk_button_new_with_label(_("Defaults"));
+	gtk_signal_connect(GTK_OBJECT(button), "clicked",
+			   GTK_SIGNAL_FUNC(filter_default_cb), NULL);
+	gtk_box_pack_end(GTK_BOX(hbox), button, FALSE, FALSE, 0);
+	gtk_widget_show(button);
 
-	add_check_button(filter_include_jpg, &filter_include_jpg_c,
-			 "JPG / JPEG", vbox2);
-	add_check_button(filter_include_xpm, &filter_include_xpm_c,
-			 "XPM", vbox2);
-	add_check_button(filter_include_tif, &filter_include_tif_c,
-			 "TIF / TIFF", vbox2);
+	button = gtk_button_new_with_label(_("Remove"));
+	gtk_signal_connect(GTK_OBJECT(button), "clicked",
+			   GTK_SIGNAL_FUNC(filter_remove_cb), NULL);
+	gtk_box_pack_end(GTK_BOX(hbox), button, FALSE, FALSE, 0);
+	gtk_widget_show(button);
 
-	vbox2 = gtk_vbox_new (FALSE, 0);
-	gtk_box_pack_start (GTK_BOX(hbox), vbox2,FALSE, FALSE, 0);
-	gtk_widget_show(vbox2);
-
-	add_check_button(filter_include_gif, &filter_include_gif_c,
-			 "GIF", vbox2);
-	add_check_button(filter_include_png, &filter_include_png_c,
-			 "PNG", vbox2);
-	add_check_button(filter_include_ppm, &filter_include_ppm_c,
-			 "PPM", vbox2);
-
-	vbox2 = gtk_vbox_new (FALSE, 0);
-	gtk_box_pack_start (GTK_BOX(hbox), vbox2,FALSE, FALSE, 0);
-	gtk_widget_show(vbox2);
-
-	add_check_button(filter_include_pgm, &filter_include_pgm_c,
-			 "PGM", vbox2);
-	add_check_button(filter_include_pcx, &filter_include_pcx_c,
-			 "PCX", vbox2);
-	add_check_button(filter_include_bmp, &filter_include_bmp_c,
-			 "BMP", vbox2);
-
-	hbox = gtk_hbox_new (FALSE, 0);
-	gtk_box_pack_start (GTK_BOX(vbox1), hbox, FALSE, FALSE, 0);
-	gtk_widget_show(hbox);
-
-	label = gtk_label_new(_("Custom file types:"));
-	gtk_box_pack_start(GTK_BOX(hbox),label,FALSE,FALSE,0);
-	gtk_widget_show(label);
-
-	custom_filter_entry = gtk_entry_new();
-	if (custom_filter) gtk_entry_set_text(GTK_ENTRY(custom_filter_entry), custom_filter);
-	gtk_box_pack_start(GTK_BOX(vbox1),custom_filter_entry,FALSE,FALSE,0);
-	gtk_widget_show(custom_filter_entry);
-
-	hbox = gtk_hbox_new (FALSE, 0);
-	gtk_box_pack_start (GTK_BOX(vbox1), hbox, FALSE, FALSE, 0);
-	gtk_widget_show(hbox);
-
-	label = gtk_label_new(_("format: [.foo;.bar]"));
-	gtk_box_pack_end(GTK_BOX(hbox),label,FALSE,FALSE,5);
-	gtk_widget_show(label);
+	button = gtk_button_new_with_label(_("Add"));
+	gtk_signal_connect(GTK_OBJECT(button), "clicked",
+			   GTK_SIGNAL_FUNC(filter_add_cb), NULL);
+	gtk_box_pack_end(GTK_BOX(hbox), button, FALSE, FALSE, 0);
+	gtk_widget_show(button);
 
 	/* collections tab */
 
