@@ -101,6 +101,7 @@ void image_sim_fill_data(ImageSimilarityData *sd, GdkPixbuf *pixbuf)
 			{
 			gint x, y;
 			gint r, g, b;
+			gint t;
 
 			if (x_small) i = (float)w / 32 * xs;
 
@@ -117,13 +118,16 @@ void image_sim_fill_data(ImageSimilarityData *sd, GdkPixbuf *pixbuf)
 					if (has_alpha) p++;
 					}
 				}
-			r /= x_inc * y_inc;
-			g /= x_inc * y_inc;
-			b /= x_inc * y_inc;
 
-			sd->avg_r[ys * 32 + xs] = r;
-			sd->avg_g[ys * 32 + xs] = g;
-			sd->avg_b[ys * 32 + xs] = b;
+			t = x_inc * y_inc;
+			r /= t;
+			g /= t;
+			b /= t;
+
+			t = ys * 32 + xs;
+			sd->avg_r[t] = r;
+			sd->avg_g[t] = g;
+			sd->avg_b[t] = b;
 
 			i += x_inc;
 			}
@@ -147,7 +151,7 @@ ImageSimilarityData *image_sim_new_from_pixbuf(GdkPixbuf *pixbuf)
 
 gfloat image_sim_compare(ImageSimilarityData *a, ImageSimilarityData *b)
 {
-	gfloat sim;
+	gint sim;
 	gint i;
 
 	if (!a || !b || !a->filled || !b->filled) return 0.0;
@@ -156,14 +160,12 @@ gfloat image_sim_compare(ImageSimilarityData *a, ImageSimilarityData *b)
 
 	for (i = 0; i < 1024; i++)
 		{
-		sim += (float)abs(a->avg_r[i] - b->avg_r[i]) / 255.0;
-		sim += (float)abs(a->avg_g[i] - b->avg_g[i]) / 255.0;
-		sim += (float)abs(a->avg_b[i] - b->avg_b[i]) / 255.0;
+		sim += abs(a->avg_r[i] - b->avg_r[i]);
+		sim += abs(a->avg_g[i] - b->avg_g[i]);
+		sim += abs(a->avg_b[i] - b->avg_b[i]);
 		}
 
-	sim /= (1024.0 * 3.0);
-
-	return 1.0 - sim;
+	return 1.0 - ((float)sim / (255.0 * 1024.0 * 3.0));
 }
 
 /* this uses a cutoff point so that it can abort early when it gets to
@@ -171,7 +173,7 @@ gfloat image_sim_compare(ImageSimilarityData *a, ImageSimilarityData *b)
  */
 gfloat image_sim_compare_fast(ImageSimilarityData *a, ImageSimilarityData *b, gfloat min)
 {
-	gfloat sim;
+	gint sim;
 	gint i;
 	gint j;
 
@@ -184,16 +186,14 @@ gfloat image_sim_compare_fast(ImageSimilarityData *a, ImageSimilarityData *b, gf
 		{
 		for (i = j; i < j + 32; i++)
 			{
-			sim += (float)abs(a->avg_r[i] - b->avg_r[i]) / 255.0;
-			sim += (float)abs(a->avg_g[i] - b->avg_g[i]) / 255.0;
-			sim += (float)abs(a->avg_b[i] - b->avg_b[i]) / 255.0;
+			sim += abs(a->avg_r[i] - b->avg_r[i]);
+			sim += abs(a->avg_g[i] - b->avg_g[i]);
+			sim += abs(a->avg_b[i] - b->avg_b[i]);
 			}
 		/* check for abort, if so return 0.0 */
-		if (sim / (1024.0 * 3.0) > min) return 0.0;
+		if ((float)sim / (255.0 * 1024.0 * 3.0) > min) return 0.0;
 		}
 
-	sim /= (1024.0 * 3.0);
-
-	return 1.0 - sim;
+	return (1.0 - ((float)sim / (255.0 * 1024.0 *3.0)) );
 }
 
