@@ -520,20 +520,21 @@ static void tab_completion_popup_list(TabCompData *td, GList *list)
 	if (event && event->type == GDK_BUTTON_RELEASE)
 		{
 		ebutton = event->button.button;
-		etime = event->button.time;
-		}
-	else if (event && event->type == GDK_KEY_PRESS)
-		{
-		ebutton = 0;
-		etime = event->key.time;
 		}
 	else
 		{
 		ebutton = 0;
-		etime = gdk_time_get();
 		}
 
-	if (event) gdk_event_free(event);
+	if (event)
+		{
+		etime = gdk_event_get_time(event);
+		gdk_event_free(event);
+		}
+	else
+		{
+		etime = gdk_time_get();
+		}
 
 	gtk_menu_popup(GTK_MENU(menu), NULL, NULL,
 		       tab_completion_popup_pos_cb, td, ebutton, etime);
