@@ -322,7 +322,7 @@ void file_dialog_add_path_widgets(FileDialog *fd, const gchar *default_path, con
 			}
 		}
 
-	list = path_selection_new_with_files(fd->entry, path, filter, filter_desc);
+	list = path_selection_new_with_files(fd->entry, fd->dest_path, filter, filter_desc);
 	path_selection_add_select_func(fd->entry, file_dialog_entry_enter_cb, fd);
 	gtk_box_pack_end(GTK_BOX(GENERIC_DIALOG(fd)->vbox), list, TRUE, TRUE, 5);
 	gtk_widget_show(list);
