@@ -342,6 +342,7 @@ static void file_util_move_multiple_skip_cb(GenericDialog *gd, gpointer data)
 	FileDataMult *fdm = data;
 
 	fdm->skip = TRUE;
+	fdm->confirmed = TRUE;
 	file_util_move_multiple(fdm);
 }
 
@@ -354,6 +355,14 @@ static void file_util_move_multiple_skip_all_cb(GenericDialog *gd, gpointer data
 	file_util_move_multiple(fdm);
 }
 
+static void file_util_move_multiple_continue_cb(GenericDialog *gd, gpointer data)
+{
+	FileDataMult *fdm = data;
+
+	fdm->confirmed = TRUE;
+	file_util_move_multiple(fdm);
+}
+
 static void file_util_move_multiple_cancel_cb(GenericDialog *gd, gpointer data)
 {
 	FileDataMult *fdm = data;
@@ -391,141 +400,79 @@ static void file_util_move_multiple_rename_cb(GtkWidget *widget, gpointer data)
 	if (fdm->rename) gtk_widget_grab_focus(fdm->rename_entry);
 }
 
+static GenericDialog *file_util_move_multiple_confirm_dialog(FileDataMult *fdm)
+{
+	GenericDialog *gd;
+	gchar *text;
+
+	GtkWidget *hbox;
+
+	text = g_strdup_printf(_("Overwrite file:\n %s\n with:\n %s"), fdm->dest, fdm->source);
+	gd = file_util_gen_dlg(_("Overwrite file"), text, "GQview", "dlg_confirm", TRUE,
+						file_util_move_multiple_cancel_cb, fdm);
+	g_free(text);
+
+	generic_dialog_add(gd, _("Yes"), file_util_move_multiple_ok_cb, TRUE);
+	fdm->yes_all_button = generic_dialog_add(gd, _("Yes to all"),
+								 file_util_move_multiple_all_cb, FALSE);
+	generic_dialog_add(gd, _("Skip"), file_util_move_multiple_skip_cb, FALSE);
+	generic_dialog_add(gd, _("Skip all"), file_util_move_multiple_skip_all_cb, FALSE);
+	generic_dialog_add_images(gd, fdm->dest, fdm->source);
+
+	/* rename option */
+
+	fdm->rename = FALSE;
+	fdm->rename_all = FALSE;
+	fdm->rename_auto = FALSE;
+
+	hbox = gtk_hbox_new(FALSE, 5);
+	gtk_box_pack_start(GTK_BOX(gd->vbox), hbox, FALSE, FALSE, 0);
+	gtk_widget_show(hbox);
+
+	fdm->rename_auto_box = gtk_check_button_new_with_label(_("Auto rename"));
+	gtk_signal_connect(GTK_OBJECT(fdm->rename_auto_box), "clicked",
+			   GTK_SIGNAL_FUNC(file_util_move_multiple_rename_auto_cb), gd);
+	gtk_box_pack_start(GTK_BOX(hbox), fdm->rename_auto_box, FALSE, FALSE, 0);
+	gtk_widget_show(fdm->rename_auto_box);
+
+	hbox = gtk_hbox_new(FALSE, 5);
+	gtk_box_pack_start(GTK_BOX(gd->vbox), hbox, FALSE, FALSE, 0);
+	gtk_widget_show(hbox);
+
+	fdm->rename_box = gtk_check_button_new_with_label(_("Rename"));
+	gtk_signal_connect(GTK_OBJECT(fdm->rename_box), "clicked",
+			   GTK_SIGNAL_FUNC(file_util_move_multiple_rename_cb), gd);
+	gtk_box_pack_start(GTK_BOX(hbox), fdm->rename_box, FALSE, FALSE, 0);
+	gtk_widget_show(fdm->rename_box);
+
+	fdm->rename_entry = gtk_entry_new();
+	gtk_entry_set_text(GTK_ENTRY(fdm->rename_entry), filename_from_path(fdm->dest));
+	gtk_widget_set_sensitive(fdm->rename_entry, FALSE);
+	gtk_box_pack_start(GTK_BOX(hbox), fdm->rename_entry, TRUE, TRUE, 0);
+	gtk_widget_show(fdm->rename_entry);
+
+	return gd;
+}
+
 static void file_util_move_multiple(FileDataMult *fdm)
 {
 	while (fdm->dest || fdm->source_next)
 		{
+		gint success = FALSE;
+		gint skip_file = FALSE;
+
 		if (!fdm->dest)
 			{
 			GList *work = fdm->source_next;
 			fdm->source = work->data;
 			fdm->dest = concat_dir_and_file(fdm->dest_base, filename_from_path(fdm->source));
 			fdm->source_next = work->next;
+			fdm->confirmed = FALSE;
 			}
 
 		if (fdm->dest && fdm->source && strcmp(fdm->dest, fdm->source) == 0)
 			{
-			GenericDialog *gd;
-			const gchar *title;
-			gchar *text;
-
-			if (fdm->copy)
-				{
-				title = _("Source to copy matches destination");
-				text = g_strdup_printf(_("Unable to copy file:\n%s\nto itself."), fdm->dest);
-				}
-			else
-				{
-				title = _("Source to move matches destination");
-				text = g_strdup_printf(_("Unable to move file:\n%s\nto itself."), fdm->dest);
-				}
-
-			gd = file_util_gen_dlg(title, text, "GQview", "dlg_confirm", TRUE,
-						file_util_move_multiple_cancel_cb, fdm);
-			g_free(text);
-			generic_dialog_add(gd, _("Continue"), file_util_move_multiple_skip_cb, TRUE);
-
-			gtk_widget_show(gd->dialog);
-			return;
-			}
-		else if (isfile(fdm->dest) && !fdm->confirmed && !fdm->confirm_all && !fdm->skip)
-			{
-			GenericDialog *gd;
-			gchar *text;
-
-			GtkWidget *hbox;
-
-			text = g_strdup_printf(_("Overwrite file:\n %s\n with:\n %s"), fdm->dest, fdm->source);
-			gd = file_util_gen_dlg(_("Overwrite file"), text, "GQview", "dlg_confirm", TRUE,
-						file_util_move_multiple_cancel_cb, fdm);
-			g_free(text);
-
-			generic_dialog_add(gd, _("Yes"), file_util_move_multiple_ok_cb, TRUE);
-			fdm->yes_all_button = generic_dialog_add(gd, _("Yes to all"),
-								 file_util_move_multiple_all_cb, FALSE);
-			generic_dialog_add(gd, _("Skip"), file_util_move_multiple_skip_cb, FALSE);
-			generic_dialog_add(gd, _("Skip all"), file_util_move_multiple_skip_all_cb, FALSE);
-			generic_dialog_add_images(gd, fdm->dest, fdm->source);
-
-			/* rename option */
-
-			fdm->rename = FALSE;
-			fdm->rename_all = FALSE;
-			fdm->rename_auto = FALSE;
-
-			hbox = gtk_hbox_new(FALSE, 5);
-			gtk_box_pack_start(GTK_BOX(gd->vbox), hbox, FALSE, FALSE, 0);
-			gtk_widget_show(hbox);
-
-			fdm->rename_auto_box = gtk_check_button_new_with_label(_("Auto rename"));
-			gtk_signal_connect(GTK_OBJECT(fdm->rename_auto_box), "clicked",
-					   GTK_SIGNAL_FUNC(file_util_move_multiple_rename_auto_cb), gd);
-			gtk_box_pack_start(GTK_BOX(hbox), fdm->rename_auto_box, FALSE, FALSE, 0);
-			gtk_widget_show(fdm->rename_auto_box);
-
-			hbox = gtk_hbox_new(FALSE, 5);
-			gtk_box_pack_start(GTK_BOX(gd->vbox), hbox, FALSE, FALSE, 0);
-			gtk_widget_show(hbox);
-
-			fdm->rename_box = gtk_check_button_new_with_label(_("Rename"));
-			gtk_signal_connect(GTK_OBJECT(fdm->rename_box), "clicked",
-					   GTK_SIGNAL_FUNC(file_util_move_multiple_rename_cb), gd);
-			gtk_box_pack_start(GTK_BOX(hbox), fdm->rename_box, FALSE, FALSE, 0);
-			gtk_widget_show(fdm->rename_box);
-
-			fdm->rename_entry = gtk_entry_new();
-			gtk_entry_set_text(GTK_ENTRY(fdm->rename_entry), filename_from_path(fdm->dest));
-			gtk_widget_set_sensitive(fdm->rename_entry, FALSE);
-			gtk_box_pack_start(GTK_BOX(hbox), fdm->rename_entry, TRUE, TRUE, 0);
-			gtk_widget_show(fdm->rename_entry);
-
-			gtk_widget_show(gd->dialog);
-			return;
-			}
-		else
-			{
-			gint success = FALSE;
-			if (fdm->skip)
-				{
-				success = TRUE;
-				if (!fdm->confirm_all) fdm->skip = FALSE;
-				}
-			else
-				{
-				gint try = TRUE;
-
-				if (fdm->confirm_all && fdm->rename_all && isfile(fdm->dest))
-					{
-					gchar *buf;
-					buf = unique_filename_simple(fdm->dest);
-					if (buf)
-						{
-						g_free(fdm->dest);
-						fdm->dest = buf;
-						}
-					else
-						{
-						try = FALSE;
-						}
-					}
-
-				if (try)
-					{
-					if (fdm->copy)
-						{
-						success = copy_file(fdm->source, fdm->dest);
-						}
-					else
-						{
-						if (move_file(fdm->source, fdm->dest))
-							{
-							success = TRUE;
-							file_maint_moved(fdm->source, fdm->dest, fdm->source_list);
-							}
-						}
-					}
-				}
-			if (!success)
+			if (!fdm->confirmed)
 				{
 				GenericDialog *gd;
 				const gchar *title;
@@ -533,26 +480,104 @@ static void file_util_move_multiple(FileDataMult *fdm)
 
 				if (fdm->copy)
 					{
-					title = _("Error copying file");
-					text = g_strdup_printf(_("Unable to copy file:\n%sto:\n%s\n during multiple file copy."), fdm->source, fdm->dest);
+					title = _("Source to copy matches destination");
+					text = g_strdup_printf(_("Unable to copy file:\n%s\nto itself."), fdm->dest);
 					}
 				else
 					{
-					title = _("Error moving file");
-					text = g_strdup_printf(_("Unable to move file:\n%sto:\n%s\n during multiple file move."), fdm->source, fdm->dest);
+					title = _("Source to move matches destination");
+					text = g_strdup_printf(_("Unable to move file:\n%s\nto itself."), fdm->dest);
 					}
+
 				gd = file_util_gen_dlg(title, text, "GQview", "dlg_confirm", TRUE,
 							file_util_move_multiple_cancel_cb, fdm);
 				g_free(text);
-
-				generic_dialog_add(gd, _("Continue"), file_util_move_multiple_skip_cb, TRUE);
+				generic_dialog_add(gd, _("Continue"), file_util_move_multiple_continue_cb, TRUE);
 
 				gtk_widget_show(gd->dialog);
 				return;
 				}
-			fdm->confirmed = FALSE;
-			g_free(fdm->dest);
-			fdm->dest = NULL;
+			skip_file = TRUE;
+			}
+		else if (isfile(fdm->dest))
+			{
+			if (!fdm->confirmed && !fdm->confirm_all)
+				{
+				GenericDialog *gd;
+
+				gd = file_util_move_multiple_confirm_dialog(fdm);
+				gtk_widget_show(gd->dialog);
+				return;
+				}
+			if (fdm->skip) skip_file = TRUE;
+			}
+
+		if (skip_file)
+			{
+			success = TRUE;
+			if (!fdm->confirm_all) fdm->skip = FALSE;
+			}
+		else
+			{
+			gint try = TRUE;
+
+			if (fdm->confirm_all && fdm->rename_all && isfile(fdm->dest))
+				{
+				gchar *buf;
+				buf = unique_filename_simple(fdm->dest);
+				if (buf)
+					{
+					g_free(fdm->dest);
+					fdm->dest = buf;
+					}
+				else
+					{
+					try = FALSE;
+					}
+				}
+
+			if (try)
+				{
+				if (fdm->copy)
+					{
+					success = copy_file(fdm->source, fdm->dest);
+					}
+				else
+					{
+					if (move_file(fdm->source, fdm->dest))
+						{
+						success = TRUE;
+						file_maint_moved(fdm->source, fdm->dest, fdm->source_list);
+						}
+					}
+				}
+			}
+
+		g_free(fdm->dest);
+		fdm->dest = NULL;
+
+		if (!success)
+			{
+			GenericDialog *gd;
+			const gchar *title;
+			gchar *text;
+
+			if (fdm->copy)
+				{
+				title = _("Error copying file");
+				text = g_strdup_printf(_("Unable to copy file:\n%sto:\n%s\n during multiple file copy."), fdm->source, fdm->dest);
+				}
+			else
+				{
+				title = _("Error moving file");
+				text = g_strdup_printf(_("Unable to move file:\n%sto:\n%s\n during multiple file move."), fdm->source, fdm->dest);
+				}
+			gd = file_util_gen_dlg(title, text, "GQview", "dlg_confirm", TRUE,
+						file_util_move_multiple_cancel_cb, fdm);
+			g_free(text);
+			generic_dialog_add(gd, _("Continue"), file_util_move_multiple_continue_cb, TRUE);
+			gtk_widget_show(gd->dialog);
+			return;
 			}
 		}
 
