@@ -158,10 +158,22 @@ static void vflist_pop_menu_info_cb(GtkWidget *widget, gpointer data)
 static void vflist_pop_menu_view_cb(GtkWidget *widget, gpointer data)
 {
 	ViewFileList *vfl = data;
-	const gchar *path;
 
-	path = vflist_index_get_path(vfl, vfl->click_row);
-	view_window_new(path);
+	if (vflist_index_is_selected(vfl, vfl->click_row))
+		{
+		GList *list;
+
+		list = vflist_selection_get_list(vfl);
+		view_window_new_from_list(list);
+		path_list_free(list);
+		}
+	else
+		{
+		const gchar *path;
+
+		path = vflist_index_get_path(vfl, vfl->click_row);
+		view_window_new(path);
+		}
 }
 
 static void vflist_pop_menu_copy_cb(GtkWidget *widget, gpointer data)
@@ -1432,6 +1444,10 @@ void vflist_maint_removed(ViewFileList *vfl, const gchar *path, GList *ignore_li
 	gtk_clist_remove(GTK_CLIST(vfl->clist), row);
 	list = g_list_nth(vfl->list, row);
 	fd = list->data;
+
+	/* thumbnail loader check */
+	if (fd == vfl->thumbs_filedata) vfl->thumbs_filedata = NULL;
+
 	vfl->list = g_list_remove(vfl->list, fd);
 	file_data_free(fd);
 
